Exception: Mustache::ContextLevelViolation

Inherits:
ContextMiss
  • Object
show all
Defined in:
lib/mustache/context_miss.rb

Overview

A ContextLevelViolation is raised whenever accessing a key in the context is not allowed by the selected context access security level

For example, if your context access security level is set to 3 but your template tries to access an inherited method this exception will be raised.

By default it is only raised for blacklisted keys in objects. See Mustache.context_access_security_level.