Exception: Mustache::ContextMiss

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/mustache/context_miss.rb

Overview

A ContextMiss is raised whenever a tag's target can not be found in the current context if Mustache#raise_on_context_miss? is set to true.

For example, if your View class does not respond to music but your template contains a {{music}} tag this exception will be raised.

By default it is not raised. See Mustache.raise_on_context_miss.

Direct Known Subclasses

ContextLevelViolation